/* ========================================
 *
 * Copyright YOUR COMPANY, THE YEAR
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF your company.
 *
 * ========================================
*/
#include "project.h"

//DMA Configration constants for TX
#define DMA_TX_BYTES_PER_BURST      1
#define DMA_TX_REQUEST_PER_BURST    1
#define DMA_TX_SRC_BASE (CYDEV_SRAM_BASE)
#define DMA_TX_DST_BASE (CYDEV_PERIPH_BASE)

//DMA Configuration constants for RX
#define DMA_RX_BYTES_PER_BURST      1
#define DMA_RX_REQUEST_PER_BURST    1
#define DMA_RX_SRC_BASE (CYDEV_PERIPH_BASE)
#define DMA_RX_DST_BASE (CYDEV_SRAM_BASE)

void Init_DMA_TX();
uint8 Init_DMA_RX_A();
uint8 Init_DMA_RX_B();

uint8 RXA_Buffer[100];
uint8 RXB_Buffer[100];

uint8 DMA_RX_A_hndl;
uint8 DMA_RX_B_hndl;

int main(void)
{
    volatile uint8 temp = 0;
    
    CyGlobalIntEnable; /* Enable global interrupts. */

    /* Place your initialization/startup code here (e.g. MyInst_Start()) */
    
    Counter_A_Start();
    Counter_B_Start();
    
    UART_Main_Start();
    UART_A_Start();
    UART_B_Start();

    Init_DMA_TX();
    DMA_RX_A_hndl = Init_DMA_RX_A();
    DMA_RX_B_hndl = Init_DMA_RX_B();
    
    for(;;)
    {
        /* Place your application code here. */
		temp = Counter_A_ReadCounter();
        if(temp >= 67)
        {
			Counter_A_Stop();
            Counter_A_WriteCounter(0);
			CyDmaChSetRequest(DMA_RX_A_hndl, CY_DMA_CPU_TERM_CHAIN);	// Force a DMA stop of all TDs for the channel
			
			
			UART_Main_PutString("[A=");
            UART_Main_PutArray(RXA_Buffer, temp);
            UART_Main_PutString("]\n\r");
            
            memset(RXA_Buffer, 0, temp);

            //To reset transition counter called DmaRelease and Init_DMA_RX
            
//            DMA_RX_A_DmaRelease();
//            DMA_RX_A_hndl = Init_DMA_RX_A();
			Counter_A_Enable();					// reenable the Counter
		    CyDmaChEnable(DMA_RX_A_hndl, 1);	// reenable the DMA with PreserveTDs
        }

		temp = Counter_B_ReadCounter();
        if(temp >= 67)
        {
			Counter_B_Stop();
            Counter_B_WriteCounter(0);
			CyDmaChSetRequest(DMA_RX_B_hndl, CY_DMA_CPU_TERM_CHAIN);	// Force a DMA stop of all TDs for the channel
			            
            UART_Main_PutString("[B="); 
            UART_Main_PutArray(RXB_Buffer, temp);
            UART_Main_PutString("]\n\r");
            
            memset(RXB_Buffer, 0, temp);
            
            //To reset transition counter called DmaRelease and Init_DMA_RX

//            DMA_RX_B_DmaRelease();          
//            Init_DMA_RX_B();
			Counter_B_Enable();					// reenable the Counter
		    CyDmaChEnable(DMA_RX_B_hndl, 1);	// reenable the DMA with PreserveTDs
        }	
	}
}

void Init_DMA_TX()
{
    uint8 DMA_TX_Chan;
    uint8 DMA_TX_TD[1];

    DMA_TX_Chan = DMA_TX_A_DmaInitialize(DMA_TX_BYTES_PER_BURST, DMA_TX_REQUEST_PER_BURST, HI16(DMA_TX_SRC_BASE), HI16(DMA_TX_DST_BASE)); //
    DMA_TX_TD[0] = CyDmaTdAllocate();
    CyDmaTdSetConfiguration(DMA_TX_TD[0], 1, DMA_TX_TD[0], DMA_TX_A__TD_TERMOUT_EN ); //
    CyDmaTdSetAddress(DMA_TX_TD[0], LO16((uint32)&UART_Main_RXDATA_REG), LO16((uint32)&UART_A_TXDATA_REG)); //
    CyDmaChSetInitialTd(DMA_TX_Chan, DMA_TX_TD[0]);
    CyDmaChEnable(DMA_TX_Chan, 1);	//  with PreserveTDs
    
    DMA_TX_Chan = DMA_TX_B_DmaInitialize(DMA_TX_BYTES_PER_BURST, DMA_TX_REQUEST_PER_BURST, HI16(DMA_TX_SRC_BASE), HI16(DMA_TX_DST_BASE)); //
    DMA_TX_TD[0] = CyDmaTdAllocate();
    CyDmaTdSetConfiguration(DMA_TX_TD[0], 1, DMA_TX_TD[0], DMA_TX_B__TD_TERMOUT_EN ); //
    CyDmaTdSetAddress(DMA_TX_TD[0], LO16((uint32)&UART_Main_RXDATA_REG), LO16((uint32)&UART_B_TXDATA_REG)); //
    CyDmaChSetInitialTd(DMA_TX_Chan, DMA_TX_TD[0]);
    CyDmaChEnable(DMA_TX_Chan, 1);	//  with PreserveTDs
}

uint8 Init_DMA_RX_A()
{
    uint8 DMA_RX_Chan;
    static uint8 DMA_RX_TD[1];
    
    DMA_RX_Chan = DMA_RX_A_DmaInitialize(DMA_RX_BYTES_PER_BURST, DMA_RX_REQUEST_PER_BURST, HI16(DMA_RX_SRC_BASE), HI16(DMA_RX_DST_BASE)); //
    DMA_RX_TD[0] = CyDmaTdAllocate();
    CyDmaTdSetConfiguration(DMA_RX_TD[0], sizeof(RXA_Buffer), DMA_RX_TD[0], CY_DMA_TD_INC_DST_ADR);
    CyDmaTdSetAddress(DMA_RX_TD[0], LO16((uint32)UART_A_RXDATA_PTR), LO16((uint32)RXA_Buffer)); //
    CyDmaChSetInitialTd(DMA_RX_Chan, DMA_RX_TD[0]);
    CyDmaChEnable(DMA_RX_Chan, 1);	//  with PreserveTDs
	return(DMA_RX_Chan);
}

uint8 Init_DMA_RX_B()
{
    uint8 DMA_RX_Chan;
    static uint8 DMA_RX_TD[1];
    
    DMA_RX_Chan = DMA_RX_B_DmaInitialize(DMA_RX_BYTES_PER_BURST, DMA_RX_REQUEST_PER_BURST, HI16(DMA_RX_SRC_BASE), HI16(DMA_RX_DST_BASE)); //
    DMA_RX_TD[0] = CyDmaTdAllocate();
    CyDmaTdSetConfiguration(DMA_RX_TD[0], sizeof(RXB_Buffer), DMA_RX_TD[0], CY_DMA_TD_INC_DST_ADR);
    CyDmaTdSetAddress(DMA_RX_TD[0], LO16((uint32)UART_B_RXDATA_PTR), LO16((uint32)RXB_Buffer)); //
    CyDmaChSetInitialTd(DMA_RX_Chan, DMA_RX_TD[0]);
    CyDmaChEnable(DMA_RX_Chan, 1);	//  with PreserveTDs
	return(DMA_RX_Chan);
}

/* [] END OF FILE */
